"""
******************************************************************************
* @author  ARMCNC site:www.armcnc.net github:armcnc.github.io
******************************************************************************
"""

import sys
import subprocess
import linuxcnc
from .status import Status
from .command import Command
from .hal import Hal
from .error import Error

class CNC:

    def __init__(self, framework):
        self.framework = framework
        self.status = Status(self)
        self.command = Command(self)
        self.hal = Hal(self)
        self.error = Error(self)

    def start(self):
        linuxcnc_pid = subprocess.Popen(["pidof", "-x", "linuxcnc"], stdout=subprocess.PIPE)
        linuxcnc_pid_result = linuxcnc_pid.communicate()[0]
        if len(linuxcnc_pid_result) == 0:
            # self.framework.utils.service.service_write({"command": "launch:restart", "message": "", "data": False})
            sys.exit()
        self.framework.machine.is_alive = True
        self.framework.utils.service.service_write({"command": "launch:restart", "message": "", "data": True})

    def message_callback(self, message):
        if message and message["command"] and message["command"] != "":
            if message["command"] == "desktop:control:device:estop":
                self.status.api.poll()
                if self.status.api.task_state == linuxcnc.STATE_ESTOP:
                    self.command.api.state(linuxcnc.STATE_ESTOP_RESET)
                else:
                    if self.status.api.task_state == linuxcnc.STATE_ESTOP_RESET or self.status.api.task_state == linuxcnc.STATE_ON or self.status.api.task_state == linuxcnc.STATE_OFF:
                        self.command.api.state(linuxcnc.STATE_ESTOP)
                self.command.api.wait_complete(0.5)

            if message["command"] == "desktop:control:start":
                self.command.on_start(message["data"]["line"])

            if message["command"] == "desktop:control:pause":
                self.command.on_pause()

            if message["command"] == "desktop:control:stop":
                self.command.on_stop()

            if message["command"] == "desktop:control:device:override_limits":
                self.command.override_limits()

            if message["command"] == "desktop:control:device:home":
                if len(self.framework.machine.axes) > 0:
                    if message["data"] == "all":
                        self.command.home_all()
                    else:
                        value = int(message["data"])
                        self.command.set_teleop_enable_mode(0)
                        self.command.home_axis(value)

            if message["command"] == "desktop:control:set:offset":
                if len(self.framework.machine.axes) > 0:
                    self.command.set_offset(message["data"])

            if message["command"] == "desktop:control:relative:offset":
                if len(self.framework.machine.axes) > 0:
                    self.command.set_axis_offset(message["data"])

            if message["command"] == "desktop:control:jog:start":
                axis = message["data"]["axis"]
                speed = message["data"]["speed"]
                jog_mode = self.command.get_jog_mode()
                if jog_mode:
                    axis = self.framework.machine.get_axes_num(axis)
                else:
                    axis = self.framework.machine.get_axis_num(axis)
                speed = speed / 60
                increment = message["data"]["increment"]
                if increment == -1:
                    self.command.jog_continuous(axis, speed, jog_mode)
                else:
                    self.command.jog_increment(axis, speed, increment, jog_mode)

            if message["command"] == "desktop:control:jog:stop":
                axis = message["data"]["axis"]
                jog_mode = self.command.get_jog_mode()
                if jog_mode:
                    axis = self.framework.machine.get_axes_num(axis)
                else:
                    axis = self.framework.machine.get_axis_num(axis)
                self.command.jog_stop(axis, jog_mode)

            if message["command"] == "desktop:control:spindle":
                value = message["data"]["value"]
                speed = message["data"]["speed"]
                if value == "on":
                    self.command.set_spindle_on(speed)
                if value == "forward":
                    self.command.set_spindle_forward(speed)
                if value == "reverse":
                    self.command.set_spindle_reverse(speed)
                if value == "faster":
                    self.command.set_spindle_faster()
                if value == "slower":
                    self.command.set_spindle_slower()
                if value == "off":
                    self.command.set_spindle_off()
                if value == "speed":
                    self.command.set_spindle_speed(speed)

            if message["command"] == "desktop:control:spindle:override":
                value = message["data"]["value"]
                self.command.set_spindle_override(value)

            if message["command"] == "desktop:control:max:velocity":
                value = message["data"]["value"]
                self.command.set_max_velocity(value)

            if message["command"] == "desktop:control:feed:rate":
                value = message["data"]["value"]
                self.command.set_feed_rate(value)

            if message["command"] == "desktop:control:device:start":
                self.status.api.poll()
                if self.status.api.task_state == linuxcnc.STATE_ESTOP:
                    return False
                if self.status.api.task_state == linuxcnc.STATE_ON:
                    self.command.api.state(linuxcnc.STATE_OFF)
                else:
                    if self.status.api.task_state == linuxcnc.STATE_OFF or self.status.api.task_state == linuxcnc.STATE_ESTOP_RESET:
                        self.command.api.state(linuxcnc.STATE_ON)
                self.command.api.wait_complete(0.5)

            if message["command"] == "desktop:control:mdi":
                value = message["data"]["value"]
                self.command.set_mdi(value)

            if message["command"] == "desktop:program:open":
                if message["data"] != "":
                    self.command.program_open(message["data"])

            if message["command"] == "service:package:status":
                self.framework.package.set_status(message["data"]["package"], message["data"]["status"])




"""
******************************************************************************
* @author  ARMCNC site:www.armcnc.net github:armcnc.github.io
******************************************************************************
"""

import configparser

class Machine:

    def __init__(self, framework):
        self.framework = framework
        self.user = "armcnc"
        self.axes = []
        self.axes_tmp = ""
        self.axis_tmp = ""
        self.is_alive = False
        self.info = None
        self.data = {"index": 0, "position": {}, "velocity": {}, "g_offset": {}, "g5x_offset": {}, "g92_offset": {}, "dtg_offset": {}, "tool": {}, "options": []}
        self.machine_path = ""
        self.workspace = "/opt/armcnc"
        self.task_state = False

    def set_data(self, index):
        self.data["index"] = index
        if len(self.data["options"]) == 0:
            for key, val in enumerate(range(9)):
                if key > 5:
                    self.data["options"].append({"label": "P" + str(key + 1) + " G59." + str((key - 6) + 1), "p_name": "P" + str(key + 1), "value": key + 1, "name": "G59." + str((key - 6) + 1)})
                else:
                    self.data["options"].append({"label": "P" + str(key + 1) + " G5" + str(key + 4), "p_name": "P" + str(key + 1), "value": key + 1, "name": "G5" + str(key + 4)})
        return self.data

    def get_data(self):
        return self.data

    def get_axes_num(self, axes):
        self.axes_tmp = ''.join(self.axes)
        num = self.axes_tmp.find(axes.upper())
        return num

    def get_axis_num(self, axis):
        self.axis_tmp = "XYZABCUVW"
        num = self.axis_tmp.find(axis.upper())
        return num

    def get_axis_name(self, num):
        self.axis_tmp = "XYZABCUVW"
        name = self.axis_tmp[num]
        return name

    def get_user_config_value(self, father, value):
        config = configparser.ConfigParser()
        config.read(self.workspace + "/configs/" + self.machine_path + "/machine.user")
        return config[father][value].strip()

    def get_user_config_array(self, father):
        config = configparser.ConfigParser()
        config.read(self.workspace + "/configs/" + self.machine_path + "/machine.user")
        items = config.items(father)
        return items

    def get_user_config_items(self, father):
        configs = {}
        config = configparser.ConfigParser()
        config.read(self.workspace + "/configs/" + self.machine_path + "/machine.user")
        items = config.items(father)
        for key, val in items:
            key = key.upper()
            configs[key] = val.strip()
        return configs
